@@ -1,7 +1,6 @@
 package Class::Slot;
-
-# TODO support code refs as slot types
-
+# ABSTRACT: Simple, efficient, comple-time class declaration
+$Class::Slot::VERSION = '0.07';
 use strict;
 use warnings;
 
@@ -12,8 +11,6 @@ use Scalar::Util qw(refaddr);
 use Filter::Simple;
 use Carp;
 
-our $VERSION = '0.06';
-
 our $DEBUG_ALL = $ENV{CLASS_SLOT_DEBUG}; # Enable debugging for all classes
 our %DEBUG;                              # Enable debugging for individual classes
 our $XS;                                 # Class::XSAccessor support
@@ -24,9 +21,10 @@ our %C3;                                 # Enable breadth-first resolution for i
 
 BEGIN {
   $DEBUG_ALL = $ENV{CLASS_SLOT_DEBUG} ? 1 : 0;
-  $XS = $ENV{CLASS_SLOT_NO_XS} ? 1 : undef;
 
-  unless (defined $XS) {
+  if ($ENV{CLASS_SLOT_NO_XS}) {
+    $XS = 0;
+  } else {
     eval 'use Class::XSAccessor';
     $XS = $@ ? 0 : 1;
   }
@@ -37,9 +35,9 @@ INIT {
 }
 
 sub import {
-  my $caller = caller;
-  my $class  = shift;
-  my $name   = shift || return;
+  my $class = shift;
+  my $name  = shift || return;
+  my ($caller, $file, $line) = caller;
 
   # Handle special parameters
   if ($name eq '-debugall') {
@@ -95,7 +93,7 @@ sub import {
       init => sub{
         # Ensure any accessor methods defined by $caller's parent class(es)
         # have been built.
-        foreach (@{ $caller . '::ISA' }) {
+        for (@{ $caller . '::ISA' }) {
           if (exists $CLASS{$_} && defined $CLASS{$_}{init}) {
             $CLASS{$_}{init}->();
           }
@@ -105,15 +103,13 @@ sub import {
         my $ctor = _build_ctor($caller);
 
         # Build accessors
-        my $acc = join "\n", map{ _build_accessor($caller, $_) }
-          keys %{ $caller->get_slots };
+        my $acc = join "\n", map{ _build_accessor($caller, $_) } keys %{ $caller->get_slots };
 
         # Build @SLOTS
         my $slots = join ' ', map{ quote_identifier($_) }
           sort keys %{ $caller->get_slots };
 
-        my $pkg  = qq{
-package $caller;
+        my $pkg  = qq{package $caller;
 no warnings 'redefine';
 no Class::Slot;
 use Carp;
@@ -128,9 +124,7 @@ $ctor
 #-------------------------------------------------------------------------------
 # Accessors
 #-------------------------------------------------------------------------------
-$acc
-
-};
+$acc};
 
         if ($DEBUG_ALL || $DEBUG{$caller}) {
           print "\n";
@@ -165,6 +159,7 @@ $acc
     # the entire class has been loaded.
     else {
       eval qq{
+# line $line "$file"
 CHECK {
   \$Class::Slot::CLASS{'$caller'}{init}->()
     if exists \$Class::Slot::CLASS{'$caller'}{init};
@@ -175,7 +170,11 @@ CHECK {
     }
   }
 
-  $CLASS{$caller}{slot}{$name} = {};
+  $CLASS{$caller}{slot}{$name} = {
+    pkg  => $caller,
+    file => $file,
+    line => $line,
+  };
 
   if (defined $type) {
     my $addr = refaddr $type;
@@ -183,7 +182,7 @@ CHECK {
     $TYPE{$addr} = $type;
   }
 
-  foreach (qw(def req rw)) {
+  for (qw(def req rw)) {
     $CLASS{$caller}{slot}{$name}{$_} = $param{$_}
       if exists $param{$_};
   }
@@ -199,76 +198,69 @@ CHECK {
 sub _build_ctor {
   my $class = shift;
 
-  my $code = qq{
-sub new \{
+  my $code = qq{sub new \{
   my \$class = shift;
 };
 
   my $has_parents = @{ $class . '::ISA' };
 
+  # Look for constructor in inheritence change
   my $can_ctor = 0;
-  foreach (@{ $class . '::ISA' }) {
+  for (@{ $class . '::ISA' }) {
     if ($_->can('new')) {
       $can_ctor = 1;
       last;
     }
   }
 
-
-  if ($has_parents && $can_ctor) {
+  if ($can_ctor) {
     $code .= "  my \$self = \$class->SUPER::new(\@_);\n";
   } else {
     $code .= "  my \$self = bless { \@_ }, \$class;\n";
   }
 
   $code .= qq{
-  # Skip type validation when called as a SUPER method from a recognized child
-  # class' constructor.
-  return \$self if ref(\$self) ne '$class'
-    && exists \$Class::Slot::CLASS{ref(\$self)};
+  # Skip type validation when called as a SUPER method from a recognized child class' constructor.
+  return \$self if ref(\$self) ne '$class' && exists \$Class::Slot::CLASS{ref(\$self)};
 };
 
   my $slots = $class->get_slots;
 
-  foreach my $name (keys %$slots) {
+  for my $name (keys %$slots) {
     my $slot  = $slots->{$name};
+    my $line  = qq{# line $slot->{line} "$slot->{file}"};
     my $req   = $slot->{req};
     my $def   = $slot->{def};
     my $type  = $TYPE{$slot->{type}} if exists $slot->{type};
     my $ident = quote_identifier($name);
 
     if ($req && !defined $def) {
-      $code .= "\n  croak '$ident is a required field' unless exists \$self->{'$ident'};\n";
+      $code .= "\n$line\n  croak '$ident is a required field' unless exists \$self->{'$ident'};\n";
     }
 
     if ($type) {
+      my $addr = refaddr $type;
       my $check = $type->can_be_inlined
         ? $type->inline_check("\$self->{'$ident'}")
-        : "\$Class::Slot::TYPE{'$slot->{type}'}->check(\$self->{'$ident'})";
+        : "\$Class::Slot::TYPE{'$addr'}->check(\$self->{'$ident'})";
 
-      $code .= qq{
-  croak '${class}::$ident did not pass validation as type $type'
-    unless !exists \$self->{'$ident'}
-        || $check;
+      $code .= qq{$line
+  croak '${class}::$ident did not pass validation as type $type' unless !exists \$self->{'$ident'} || $check;
 
 };
     }
 
     if (defined $def) {
-      $code .= "  \$self->{'$ident'} = ";
-
-      if (ref $def eq 'CODE') {
-        $code .= "\$CLASS{'$class'}{slot}{'$ident'}{def}->(\$self)";
-      }
-      else {
-        $code .= "\$CLASS{'$class'}{slot}{'$ident'}{def}";
-      }
+      $code .= "$line\n  \$self->{'$ident'} = ";
+      $code .= (ref $def eq 'CODE')
+        ? "\$CLASS{'$class'}{slot}{'$ident'}{def}->(\$self)"
+        : "\$CLASS{'$class'}{slot}{'$ident'}{def}";
 
       $code .= " unless exists \$self->{'$ident'};\n";
     }
   }
 
-  $code .= "\n  \$self;\n}\n";
+  $code .= "  \$self;\n}\n";
 
   return $code;
 }
@@ -304,17 +296,17 @@ sub get_slots {
   my @mro = get_mro $class;
   my %slots;
 
-  foreach my $class (@mro) {
+  for my $class (@mro) {
     next unless exists $CLASS{$class};
 
     my @slots = defined $name ? ($name) : @{$CLASS{$class}{slots}};
 
-    foreach my $slot (@slots) {
+    for my $slot (@slots) {
       if (!exists $slots{$slot}) {
         $slots{$slot} = $CLASS{$class}{slot}{$slot};
       }
       else {
-        foreach my $cfg (qw(rw req def)) {
+        for my $cfg (qw(rw req def line file)) {
           if (!exists $slots{$slot}{$cfg} && exists $CLASS{$class}{slot}{$slot}{$cfg}) {
             $slots{$slot}{$cfg} = $CLASS{$class}{slot}{$slot}{$cfg};
           }
@@ -365,13 +357,12 @@ sub _build_getter_xs {
 sub _build_getter_pp {
   my ($class, $name) = @_;
   my $ident = quote_identifier($name);
-  return qq{
-sub $ident \{
-  croak "${class}::$ident is protected"
-    if \@_ > 1;
-
-  return \$_[0]->{'$ident'}
-    if defined wantarray;
+  my $slot  = $class->get_slots($name);
+  my $line  = qq{# line $slot->{line} "$slot->{file}"};
+  return qq{sub $ident \{
+$line
+  croak "${class}::$ident is protected" if \@_ > 1;
+  return \$_[0]->{'$ident'} if defined wantarray;
 \}
 };
 }
@@ -397,28 +388,28 @@ sub _build_setter_xs {
 sub _build_setter_pp {
   my ($class, $name) = @_;
   my $slot  = $class->get_slots($name);
+  my $line  = qq{# line $slot->{line} "$slot->{file}"};
   my $type  = $TYPE{$slot->{type}} if $slot->{type};
   my $ident = quote_identifier($name);
 
   my $code = "sub $ident {\n  if (\@_ > 1) {";
 
   if ($type) {
+    my $addr = refaddr $type;
     my $check = $type->can_be_inlined
       ? $type->inline_check('$_[1]')
-      : "\$Class::Slot::TYPE{'$type'}->check(\$_[1])";
+      : "\$Class::Slot::TYPE{'$addr'}->check(\$_[1])";
 
       $code .= qq{
-    croak '${class}::$ident did not pass validation as type $type'
-      unless $check;
+$line
+    croak '${class}::$ident did not pass validation as type $type' unless $check;
 };
   }
 
-  $code .= qq{
-    \$_[0]->{'$ident'} = \$_[1];
+  $code .= qq{    \$_[0]->{'$ident'} = \$_[1];
   \}
 
-  return \$_[0]->{'$ident'}
-    if defined wantarray;
+  return \$_[0]->{'$ident'} if defined wantarray;
 \}
 };
 }
@@ -465,13 +456,14 @@ sub install_method {
 FILTER {
   s/\buse slot\b/use Class::Slot/g;
   s/\bslot::/Class::Slot::/g;
-  s/^slot\b/use Class::Slot/gsm;
+  s/^\s*slot\b/use Class::Slot/gsm;
 };
 
 1;
 
-package Class::Slot::AnonType;
 
+package Class::Slot::AnonType;
+$Class::Slot::AnonType::VERSION = '0.07';
 use strict;
 use warnings;
 use Carp;
@@ -486,16 +478,28 @@ sub new {
 
 sub can_be_inlined { 0 }
 sub inline_check { croak 'not supported' }
-sub check { goto $_[0] }
+
+sub check {
+  my $self = shift;
+  $self->(shift);
+}
 
 1;
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
 Class::Slot - Simple, efficient, comple-time class declaration
 
+=head1 VERSION
+
+version 0.07
+
 =head1 SYNOPSIS
 
   package Point;
@@ -571,7 +575,19 @@ are equivalent:
   slot x => Int;
 
 A simple source filter is used to translate uses of C<slot> and C<use slot>
-into C<use Class::Slot>.
+into C<use Class::Slot>. This is a somewhat brittle solution to ensuring
+compile time code generation while avoiding a clash with
+L<Tie::Hash::KeysMask>, which uses the C<slot> namespace internally but
+nevertheless holds the keys to it on CPAN.
+
+As a result, care must be taken when defining slots using the C<slot name ...>
+syntax (rather than C<use Class::Slot name ...>). The source filter identifies
+the keyword C<slot> when it appears as the first value on a line, followed by
+a word boundary. There is the potential for false positives, such as with:
+
+  my @ots = qw(
+    slot blot glot clot
+  );
 
 =head1 OPTIONS
 
@@ -658,10 +674,10 @@ C<Class::Slot> is designed to be fast and have a low overhead. When available,
 L<Class::XSAccessor> is used to generate the class accessors. This applies to
 slots that are not writable or are writable but have no declared type.
 
-This behavior can be disabled by setting C<$Class::Slot::XS> to a negative
-value, although this must be done in a C<BEGIN> block before declaring any
-slots, or by setting the environmental variable C<CLASS_SLOT_NO_XS> to a
-positive value before running.
+This behavior can be disabled by setting C<$Class::Slot::XS> to a falsey value,
+although this must be done in a C<BEGIN> block before declaring any slots, or
+by setting the environmental variable C<CLASS_SLOT_NO_XS> to a truthy value
+before the module is loaded.
 
 A minimal benchmark on my admittedly underpowered system compares L<Moose>,
 L<Moo>, and L<Class::Slot>. The test includes multiple setters using a mix of
@@ -687,7 +703,7 @@ Jeff Ober <sysread@fastmail.fm>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2018 by Jeff Ober.
+This software is copyright (c) 2020 by Jeff Ober.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
